@@ -176,7 +176,7 @@ func (t *TargetBuilder) execExtCmds(sf stage.StageFunc, userSrcDir string,
 	}
 	defer os.Chdir(pwd)
 
-	util.StatusMessage(util.VERBOSITY_DEFAULT, "Executing %s\n", sf.Name)
+	util.StatusMessage(util.VERBOSITY_DEFAULT, "Executing %s/%s\n", sf.Pkg.FullName(), sf.Name)
 	if err := util.ShellInteractiveCommand(toks, env, true); err != nil {
 		return err
 	}
@@ -189,6 +189,7 @@ func getLinkTableEntry(name string) string {
 
 	entry := indent + "__" + name + "_start__ = .;\n" +
 		indent + "KEEP(*(." + name + "))\n" +
+		indent + "KEEP(*(SORT(." + name + ".*)))\n" +
 		indent + "__" + name + "_end__ = .;\n\n"
 
 	return entry
