@@ -74,11 +74,11 @@ type MfgBuildMeta struct {
 
 // Can be used to construct an Mfg object.
 type MfgBuilder struct {
-	BasePkg *pkg.LocalPackage
-	Bsp     *pkg.BspPackage
-	Targets []MfgBuildTarget
-	Raws    []MfgBuildRaw
-	Meta    *MfgBuildMeta
+	BasePkg     *pkg.LocalPackage
+	Bsp         *pkg.BspPackage
+	Targets     []MfgBuildTarget
+	Raws        []MfgBuildRaw
+	Meta        *MfgBuildMeta
 	BaseAddress int
 }
 
@@ -163,7 +163,7 @@ func calcBsp(dm DecodedMfg,
 		return nil, util.FmtNewtError("multiple BSPs detected")
 	}
 
-	bsp, err := pkg.NewBspPackage(bspLpkg)
+	bsp, err := pkg.NewBspPackage(bspLpkg, nil)
 	if err != nil {
 		return nil, util.FmtNewtError(err.Error())
 	}
@@ -220,16 +220,10 @@ func newMfgBuildTarget(dt DecodedTarget,
 		return MfgBuildTarget{}, err
 	}
 
-	mpath := builder.ManifestPath(dt.Name, builder.BUILD_NAME_APP,
-		t.App().Name())
+	mpath := builder.ManifestPath(dt.Name, builder.BUILD_NAME_APP, t.App().FullName())
 	man, err := manifest.ReadManifest(mpath)
 	if err != nil {
-		mpath = builder.ManifestPath(dt.Name, builder.BUILD_NAME_BOOT,
-			t.App().Name())
-		man, err = manifest.ReadManifest(mpath)
-		if err != nil {
-			return MfgBuildTarget{}, util.FmtNewtError("%s", err.Error())
-		}
+		return MfgBuildTarget{}, util.FmtNewtError("%s", err.Error())
 	}
 
 	isBoot := parse.ValueIsTrue(man.Syscfg["BOOT_LOADER"])
