@@ -24,6 +24,7 @@ import (
 	"os"
 	"path"
 	"path/filepath"
+	"regexp"
 	"strings"
 
 	log "github.com/sirupsen/logrus"
@@ -44,6 +45,7 @@ import (
 var globalProject *Project = nil
 
 const PROJECT_FILE_NAME = "project.yml"
+const PATCHES_DIR = "patches"
 
 var ignoreSearchDirs []string = []string{
 	"bin",
@@ -69,6 +71,14 @@ type Project struct {
 	//                read.
 	repos deprepo.RepoMap
 
+	// Contains names of repositories that will be upgraded.
+	// If it's empty all repos are allowed.
+	reposAllowedRe []*regexp.Regexp
+
+	// Contains names of repositories that will be excluded from upgrade.
+	// Can override repositories from reposAllowed.
+	reposIgnoredRe []*regexp.Regexp
+
 	// The local repository at the top-level of the project.  This repo is
 	// excluded from most repo operations.
 	localRepo *repo.Repo
@@ -95,7 +105,7 @@ func initProject(dir string, download bool) error {
 
 	if download {
 		err = globalProject.UpgradeIf(newtutil.NewtForce, newtutil.NewtAsk,
-			                      func(r *repo.Repo) bool { return !r.IsExternal(r.Path()) })
+			func(r *repo.Repo) bool { return !r.IsExternal(r.Path()) })
 		if err != nil {
 			return err
 		}
@@ -172,9 +182,31 @@ func NewProject(dir string, download bool) (*Project, error) {
 	return proj, nil
 }
 
-func (proj *Project) isRepoAdded(r *repo.Repo) bool {
+func (proj *Project) patternsMatch(patterns *[]*regexp.Regexp, repoName string) bool {
+	for _, re := range *patterns {
+		if re.MatchString(repoName) {
+			return true
+		}
+	}
+
+	return false
+}
+
+func (proj *Project) isRepoIgnored(repoName string) bool {
+	return proj.patternsMatch(&proj.reposIgnoredRe, repoName)
+}
+
+func (proj *Project) isRepoAllowed(repoName string) bool {
+	if (len(proj.reposAllowedRe) == 0) || proj.patternsMatch(&proj.reposAllowedRe, repoName) {
+		return !proj.patternsMatch(&proj.reposIgnoredRe, repoName)
+	}
+
+	return false
+}
+
+func (proj *Project) isRepoAdded(repoName string) bool {
 	for _, pr := range proj.repos {
-		if pr.Name() == r.Name() {
+		if pr.Name() == repoName {
 			return true
 		}
 	}
@@ -182,7 +214,6 @@ func (proj *Project) isRepoAdded(r *repo.Repo) bool {
 }
 
 func (proj *Project) GetPkgRepos() error {
-
 	for _, pkgList := range proj.packages {
 		for _, pkg := range *pkgList {
 			if pkg.PkgConfig().HasKey("repository") {
@@ -192,6 +223,10 @@ func (proj *Project) GetPkgRepos() error {
 						fields, err := pkg.PkgConfig().GetValStringMapString(k, nil)
 						util.OneTimeWarningError(err)
 
+						if !proj.isRepoAllowed(repoName) {
+							continue
+						}
+
 						r, err := proj.loadRepo(repoName, fields)
 						if err != nil {
 							// if `repository.yml` does not exist, it is not an error; we
@@ -200,6 +235,10 @@ func (proj *Project) GetPkgRepos() error {
 								return err
 							}
 						}
+						if r == nil {
+							continue
+						}
+
 						verReq, err := newtutil.ParseRepoVersion(fields["vers"])
 						if err != nil {
 							return util.FmtNewtError(
@@ -209,12 +248,27 @@ func (proj *Project) GetPkgRepos() error {
 						}
 						r.SetPkgName(pkg.Name())
 
-						if !proj.isRepoAdded(r) {
+						if !proj.isRepoAdded(r.Name()) {
 							if err := proj.addRepo(r, true); err != nil {
 								return err
 							}
 							proj.rootRepoReqs[repoName] = verReq
 						}
+
+						if _, err := os.Stat(pkg.BasePath() + "/" + PATCHES_DIR + "/" + r.Name()); os.IsNotExist(err) {
+							continue
+						} else {
+							dirEntries, err := os.ReadDir(pkg.BasePath() + "/" + PATCHES_DIR + "/" + r.Name())
+							if err != nil {
+								return err
+							}
+
+							for _, e := range dirEntries {
+								if strings.HasSuffix(e.Name(), ".patch") {
+									r.AddPatch(pkg.BasePath() + "/" + PATCHES_DIR + "/" + r.Name() + "/" + e.Name())
+								}
+							}
+						}
 					}
 				}
 			}
@@ -223,6 +277,19 @@ func (proj *Project) GetPkgRepos() error {
 	return nil
 }
 
+func (proj *Project) SetGitEnvVariables() error {
+	err := os.Setenv("GIT_COMMITTER_NAME", "newt")
+	if err != nil {
+		return err
+	}
+
+	err = os.Setenv("GIT_COMMITTER_EMAIL", "dev@mynewt.apache.org")
+	if err != nil {
+		return err
+	}
+	return nil
+}
+
 func (proj *Project) Path() string {
 	return proj.BasePath
 }
@@ -374,11 +441,10 @@ func (proj *Project) InfoIf(predicate func(r *repo.Repo) bool,
 // @param name                  The name of the repo to read.
 // @param fields                Fields containing the basic repo description.
 //
-// @return *Repo                The fully-read repo on success; nil on failure.
+// @return *Repo                The fully-read repo on success; nil on failure or when repo is not allowed
 // @return error                Error on failure.
 func (proj *Project) loadRepo(name string, fields map[string]string) (
 	*repo.Repo, error) {
-
 	// First, read the repo description from the supplied fields.
 	if fields["type"] == "" {
 		return nil,
@@ -461,6 +527,11 @@ func (proj *Project) loadRepoDeps(download bool) error {
 					depRepo := proj.repos[dep.Name]
 					if depRepo == nil {
 						var err error
+
+						if !proj.isRepoAllowed(dep.Name) {
+							continue
+						}
+
 						depRepo, err = proj.loadRepo(dep.Name, dep.Fields)
 						if err != nil {
 							// if `repository.yml` does not exist, it is not an
@@ -469,6 +540,9 @@ func (proj *Project) loadRepoDeps(download bool) error {
 								return nil, err
 							}
 						}
+						if depRepo == nil {
+							continue
+						}
 						if err := proj.addRepo(depRepo, download); err != nil {
 							return nil, err
 						}
@@ -510,13 +584,33 @@ func (proj *Project) downloadRepositoryYmlFiles() error {
 	// Download the `repository.yml` file for each root-level repo (those
 	// specified in the `project.yml` file).
 	for _, r := range proj.repos.Sorted() {
-		if !r.IsLocal() && !r.IsExternal(r.Path()) {
-			if _, err := r.UpdateDesc(); err != nil {
-				return err
+		if r.IsUpdated() {
+			continue
+		}
+
+		if r.IsLocal() {
+			continue
+		}
+
+		if r.IsExternal(r.Path()) {
+			ver := proj.rootRepoReqs[r.Name()]
+
+			// External repositories can only use commit stability since they do
+			// not have repository.yml
+			if len(ver.Commit) == 0 {
+				return util.FmtNewtError(
+					"External repository \"%s\" does not specify valid commit version (%s)",
+					r.Name(), ver.String())
+			}
+
+			// No need to fetch if requested commit is already checked out
+			if r.IsHeadCommit(ver.Commit) {
+				continue
 			}
 		}
-		if r.IsExternal(r.Path()) {
-			r.Downloader().Fetch(r.Path())
+
+		if _, err := r.UpdateDesc(); err != nil {
+			return err
 		}
 	}
 
@@ -587,6 +681,36 @@ func (proj *Project) addRepo(r *repo.Repo, download bool) error {
 	return nil
 }
 
+func (proj *Project) createRegexpPatterns(patterns []string) ([]*regexp.Regexp, error) {
+	var ret []*regexp.Regexp
+	var errLines []string
+
+	for _, pattern := range patterns {
+		var s string
+
+		if strings.HasPrefix(pattern, "~") {
+			s = "^" + pattern[1:]
+		} else if strings.HasSuffix(pattern, "*") {
+			s = "^" + pattern[:len(pattern)-1] + ".*$"
+		} else {
+			s = "^" + pattern + "$"
+		}
+
+		re, err := regexp.Compile(s)
+		if err != nil {
+			errLines = append(errLines, fmt.Sprintf("Invalid pattern: %s", pattern))
+		} else {
+			ret = append(ret, re)
+		}
+	}
+
+	if len(errLines) > 0 {
+		return ret, util.NewNewtError(strings.Join(errLines, "\n"))
+	} else {
+		return ret, nil
+	}
+}
+
 func (proj *Project) loadConfig(download bool) error {
 	yc, err := config.ReadFile(proj.BasePath + "/" + PROJECT_FILE_NAME)
 	if err != nil {
@@ -600,6 +724,24 @@ func (proj *Project) loadConfig(download bool) error {
 	proj.name, err = yc.GetValString("project.name", nil)
 	util.OneTimeWarningError(err)
 
+	var reposAllowed []string
+	var reposIgnored []string
+
+	reposAllowed, err = yc.GetValStringSlice("project.repositories.allowed", nil)
+	util.OneTimeWarningError(err)
+	proj.reposAllowedRe, err = proj.createRegexpPatterns(util.UniqueStrings(reposAllowed))
+	util.OneTimeWarningError(err)
+
+	reposIgnored, err = yc.GetValStringSlice("project.repositories.ignored", nil)
+	util.OneTimeWarningError(err)
+	reposIgnored = append(reposIgnored, newtutil.NewtIgnore...)
+	proj.reposIgnoredRe, err = proj.createRegexpPatterns(util.UniqueStrings(reposIgnored))
+	util.OneTimeWarningError(err)
+
+	if proj.isRepoIgnored("apache-mynewt-core") {
+		return util.NewNewtError("apache-mynewt-core repository cannot be on ignored list.")
+	}
+
 	// Local repository always included in initialization
 	r, err := repo.NewLocalRepo(proj.name)
 	if err != nil {
@@ -620,6 +762,10 @@ func (proj *Project) loadConfig(download bool) error {
 			fields, err := yc.GetValStringMapString(k, nil)
 			util.OneTimeWarningError(err)
 
+			if proj.isRepoIgnored(repoName) {
+				continue
+			}
+
 			r, err := proj.loadRepo(repoName, fields)
 			if err != nil {
 				// if `repository.yml` does not exist, it is not an error; we
@@ -628,6 +774,10 @@ func (proj *Project) loadConfig(download bool) error {
 					return err
 				}
 			}
+			if r == nil {
+				continue
+			}
+
 			verReq, err := newtutil.ParseRepoVersion(fields["vers"])
 			if err != nil {
 				return util.FmtNewtError(
@@ -635,7 +785,7 @@ func (proj *Project) loadConfig(download bool) error {
 						"%s (%s)",
 					repoName, fields["vers"], err.Error())
 			}
-			r.SetIsFromProjectYml()
+
 			if err := proj.addRepo(r, download); err != nil {
 				return err
 			}
