@@ -30,6 +30,7 @@ import (
 	"os/exec"
 	"os/signal"
 	"path/filepath"
+	"reflect"
 	"runtime"
 	"sort"
 	"strconv"
@@ -70,6 +71,19 @@ const (
 	VERBOSITY_VERBOSE = 3
 )
 
+type StaticLib struct {
+	File      string
+	WholeArch bool
+}
+
+func NewStaticLib(file string, wholeArch bool) StaticLib {
+	s := StaticLib{
+		File:      file,
+		WholeArch: wholeArch,
+	}
+	return s
+}
+
 func (se *NewtError) Error() string {
 	return se.Text
 }
@@ -668,6 +682,21 @@ func UniqueStrings(elems []string) []string {
 	return result
 }
 
+// Removes all duplicate static lib from the specified array, while preserving
+// order.
+func UniqueStaticLib(libs []StaticLib) []StaticLib {
+	set := make(map[StaticLib]bool)
+	result := make([]StaticLib, 0)
+
+	for _, lib := range libs {
+		if !set[lib] {
+			result = append(result, lib)
+			set[lib] = true
+		}
+	}
+	return result
+}
+
 // Sorts whitespace-delimited lists of strings.
 //
 // @param wsSepStrings          A list of strings; each string contains one or
@@ -998,3 +1027,20 @@ func DirsAreEqual(dira string, dirb string) (bool, error) {
 
 	return true, nil
 }
+
+// Function that checks if the slice of any type contains a
+// specified value
+func SliceContains(slice interface{}, elem interface{}) bool {
+	s := reflect.ValueOf(slice)
+
+	if s.Kind() != reflect.Slice {
+		panic("SliceContains() called with non-slice type")
+	}
+
+	for i := 0; i < s.Len(); i++ {
+		if reflect.DeepEqual(s.Index(i).Interface(), elem) {
+			return true
+		}
+	}
+	return false
+}
